﻿-- Localization
local L = LibStub("AceLocale-3.0"):GetLocale("TreasuresOfWoW")

local addon, TOW = ...
TOW.Version = "7.0.0"

local arrowAvailable, arrowTextAvailable = false, false

local UnitPosition, IsQuestFlaggedCompleted = UnitPosition, IsQuestFlaggedCompleted
local GetCurrentMapDungeonLevel, GetCurrentMapZone, GetCurrentMapAreaID, GetAreaMapInfo, GetPOITextureCoords = GetCurrentMapDungeonLevel, GetCurrentMapZone, GetCurrentMapAreaID, GetAreaMapInfo, GetPOITextureCoords
local FactionGroup = UnitFactionGroup("player")
local abs, format, select, pairs = math.abs, format, select, pairs

-- Default Settings
defaultDB = {
	Version = TOW.Version,
	enabled = true,
	debug = false,
	arrowEnabled = false,
	ignoretm = false,
	minimapAuto = false,
	minimap = true,
	tomtomKey = "S",
	hidden = { },
	mapIDhidden = {[941] = true,[945] = true,[946] = true,[947] = true,[948] = true,[949] = true,[950] = true,[1015] = true,[1017] = true,[1018] = true,[1024] = true,[1033] = true,}
}
local towDB = defaultDB

local function checkVersion()
	local db = TreasuresOfWoWDB
	if (db and (not db.Version or db.Version ~= TOW.Version)) then
		db.Version = TOW.Version
		if (db.tomtomKey == nil) then
			db.tomtomKey = towDB.tomtomKey
		end
		if (db.minimapAuto == nil) then
			db.minimapAuto = towDB.minimapAuto
		end
		if (db.mapIDhidden == nil) then
			db.mapIDhidden = towDB.mapIDhidden
		end
	end
end

local TreasureContinentsIds = {
	[1116] = 1116, -- Draenor
    [1152] = 1116, -- Garrison Level 1 (H)
    [1330] = 1116, -- Garrison Level 2 (H)
    [1153] = 1116, -- Garrison Level 3 (H)
    [1158] = 1116, -- Garrison Level 1 (A)
    [1331] = 1116, -- Garrison Level 2 (A)
    [1159] = 1116, -- Garrison Level 3 (A)
    [1464] = 1116, -- Tanaan Jungle
	[1220] = 1220, -- Broken Isles
}

local TreasureQuestsDesc = {
	[36466] = L["Talador"],
    [36467] = L["Spires of Arak"],
    [36465] = L["Gorgrond"],
    [34557] = L["Frostfire"],
    [36468] = L["Nagrand"],
    [36464] = L["Shadowmoon Valley"],
    [36249] = L["requires Lumber Yard"],
    [36250] = L["requires Lumber Yard"],
    [36251] = L["requires Sparring Arena"],
    [36252] = L["requires Sparring Arena"],
    [39463] = L["Tanaan Jungle"],
	[1015] = L["Azsuna"], -- Azsuna
	[1017] = L["Stormheim"], -- Tornheim
	[1018] = L["Val'sharah"], -- Val'sharah
	[1024] = L["Highmountain"], -- Haut-roc
	[1033] = L["Suramar"], -- Suramar
}
   
local frame = CreateFrame("Frame")

local function GetDistanceToPoint(pointx, pointy, x, y)
    local dX = pointx - x
    local dY = pointy - y
    return (dX * dX + dY * dY) ^ 0.5
end

local function SelectQuest(treasure, type)
    if treasure.flags == 131073 then
        if type == "map" then
            return treasure.quests[1]
        elseif type == "complete" then
            return treasure.quests[2]
        else
            return 0
        end
    elseif treasure.flags == 65541 then
        if type == "map" then
            return treasure.quests[2]
        elseif type == "complete" then
            return treasure.quests[1]
        else
            return 0
        end
    elseif treasure.flags == 65537 then
        if type == "map" then
            return treasure.quests[2]
        elseif type == "complete" then
            return treasure.quests[1]
        else
            return 0
        end
    elseif treasure.flags == 524310 then
        if type == "choise1" then
            return treasure.quests[1]
        elseif type == "choise2" then
            return treasure.quests[2]
        elseif type == "map" then
            return treasure.quests[3]
        elseif type == "complete" then
            return treasure.quests[4]
        else
            return 0
        end
    elseif treasure.flags == 0 then
        return 0
    else
        return 0
    end
end

local function GetTreasureCompletion(treasure)
    local trquest, tmquest, c1quest, c2quest = SelectQuest(treasure, "complete"), SelectQuest(treasure, "map"), SelectQuest(treasure, "choise1"), SelectQuest(treasure, "choise2")
    local trCheck = trquest == 0 or not IsQuestFlaggedCompleted(trquest)
    local tmCheck = TreasuresOfWoWDB.ignoretm or tmquest == 0 or not IsQuestFlaggedCompleted(tmquest)
    local choise1Check = c1quest == 0 or IsQuestFlaggedCompleted(c1quest)
    local choise2Check = c2quest == 0 or IsQuestFlaggedCompleted(c2quest)
    return trCheck, tmCheck, choise1Check, choise2Check, trquest, tmquest, c1quest, c2quest
end

function GetClosestTreasure()
    local dist = 99999999
    local closest = nil
    local x, y, _, mapID = UnitPosition("player")
    for k,v in pairs(TOW.treasures) do
		if TreasureContinentsIds[mapID] == v.continent then
			local dist2 = GetDistanceToPoint(x, y, v.x, v.y)
			local trCheck, tmCheck, choise1Check, choise2Check = GetTreasureCompletion(v)
			local factionTreasure = (v.faction == FactionGroup or v.faction == nil)
			local GarrisonLvl = C_Garrison.GetGarrisonInfo(LE_GARRISON_TYPE_6_0)
			local GarrisonLvlTreasure = (v.garrisonLvl == GarrisonLvl or v.garrisonLvl == nil)
			if dist2 < dist and factionTreasure and GarrisonLvlTreasure and TreasuresOfWoWDB.mapIDhidden[v.mapID] and (TreasuresOfWoWDB.debug or not TreasuresOfWoWDB.hidden[v.id]) and (TreasuresOfWoWDB.debug or (trCheck and tmCheck and choise1Check and choise2Check)) then
				dist = dist2
				closest = v
			end
		end
	end
    return closest, dist
end

if DBM then
    arrowAvailable = true
    if DBMArrow then
        local text = DBMArrow:CreateFontString(nil, "OVERLAY", "GameTooltipText")
        text:SetWidth(150)
        text:SetHeight(60)
        text:SetPoint("CENTER", DBMArrow, "CENTER", 0, -35)
        text:SetTextColor(1, 1, 1, 1)
        text:SetJustifyH("CENTER")
        DBMArrow.text = text
        arrowTextAvailable = true
    end
end

local function ShowArrow(x, y, text)
    if arrowAvailable then
        DBM.Arrow:ShowRunTo(x, y, nil, 5)
    end
    if arrowTextAvailable then
        if not DBMArrow.text:IsShown() then
            DBMArrow.text:Show()
        end
        DBMArrow.text:SetText(text)
    end
end

local function HideArrow()
    if arrowAvailable then
        DBM.Arrow:Hide()
    end
    if arrowTextAvailable then
        DBMArrow.text:Hide()
    end
end

function TOW:DBMArrow()
    if not arrowAvailable then
		DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : arrow not available. Please install DBM in order to use arrow."])
		return
	end
	TreasuresOfWoWDB.arrowEnabled = not TreasuresOfWoWDB.arrowEnabled
	DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : arrow "]..(TreasuresOfWoWDB.arrowEnabled and L["enabled"] or L["disabled"]))
	if not TreasuresOfWoWDB.arrowEnabled then
		HideArrow()
	end
end

local function FilterMinimapPOI(v, x, y, mapID, radius)
    local showTreasures = TreasuresOfWoWDB.enabled and TreasureContinentsIds[mapID] == v.continent
    if not showTreasures then return false end

    local distCheck = GetDistanceToPoint(x, y, v.x, v.y) < radius
    if not distCheck then return false end

    local trCheck, tmCheck, choise1Check, choise2Check, _, _, _, _ = GetTreasureCompletion(v)
    if TreasuresOfWoWDB.debug or (not TreasuresOfWoWDB.hidden[v.id] and trCheck and tmCheck and choise1Check and choise2Check) then
        return true
    end
    return false
end

local interval = 1
local currentmapid = -1

local function onUpdate(self,elapsed)
    if not TreasuresOfWoWDB.enabled then return end
    if not TreasuresOfWoWDB.arrowEnabled then return end
    currentmapid = select(4, UnitPosition("player"))
    if not TreasureContinentsIds[currentmapid] then return end
    interval = interval + elapsed
    if interval >= 1 then
        interval = 0
        local closest, dist = GetClosestTreasure()
        if not closest then
            HideArrow()
            return
        end
        if TreasuresOfWoWDB.debug then
            ShowArrow(closest.x,closest.y, format("%s (%u)\n%.02f yd", closest.name, closest.id, dist))
        else
            ShowArrow(closest.x,closest.y, format("%s\n%.02f yd", closest.name, dist))
        end
    end
end

local function onEvent(self, event, ...)
    if event == "ADDON_LOADED" then
        local addonName = ...
        if addonName ~= addon then return end
		if TreasuresOfWoWDB then
			checkVersion()
		else
			TreasuresOfWoWDB = towDB
		end
        if not TreasuresOfWoWDB.hidden then
            TreasuresOfWoWDB.hidden = { }
        end
        if TreasuresOfWoWDB.minimapAuto and not TreasuresOfWoWDB.minimap then
            TreasuresOfWoWDB.minimap = true
        end
        if TreasuresOfWoWDB.enabled then
            frame:SetScript("OnUpdate", onUpdate)
        else
            frame:SetScript("OnUpdate", nil)
        end
        MinimapPOIHandler:SetFilter(FilterMinimapPOI)
        MinimapPOIHandler:SetDataSource(TOW.treasures, TreasureContinentsIds)
        if TreasuresOfWoWDB.minimap then
            MinimapPOIHandler:Enable()
        end
		-- Add options to Bliz addon config frame
		TOW:LoadOptions()
    end
end

frame:SetScript("OnEvent", onEvent)
frame:RegisterEvent("ADDON_LOADED")

SLASH_TOW1 = "/tow"

SlashCmdList["TOW"] = function(msg, editBox)
    if msg:lower() == "on" then
        frame:SetScript("OnUpdate", onUpdate)
        TreasuresOfWoWDB.enabled = true
        DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW enabled"])
    elseif msg:lower() == "off" then
        frame:SetScript("OnUpdate", nil)
        TreasuresOfWoWDB.enabled = false
        HideArrow()
        DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW disabled"])
    elseif msg:lower() == "toggle" then
        TreasuresOfWoWDB.enabled = not TreasuresOfWoWDB.enabled
        frame:SetScript("OnUpdate", (TreasuresOfWoWDB.enabled and onUpdate or nil))
        if not TreasuresOfWoWDB.enabled then HideArrow() end
        DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW "]..(TreasuresOfWoWDB.enabled and L["enabled"] or L["disabled"]))
    elseif msg:lower() == "arrow" then
        TOW:DBMArrow()
    elseif msg:lower() == "debug" then
        TreasuresOfWoWDB.debug = not TreasuresOfWoWDB.debug
        DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : debug "]..(TreasuresOfWoWDB.debug and L["enabled"] or L["disabled"]))
    elseif msg:lower() == "closest" then
        local closest, dist = GetClosestTreasure()
        if closest then
            DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : "]..closest.id.." ("..closest.name..") "..dist..L[" yd away"])
        else
            DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : there's no treasure nearby"])
        end
    elseif msg:lower() == "minimap" then
        TreasuresOfWoWDB.minimap = not TreasuresOfWoWDB.minimap
        if TreasuresOfWoWDB.minimap then
            MinimapPOIHandler:Enable()
        else
            MinimapPOIHandler:Disable()
        end
        DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : minimap icons "]..(TreasuresOfWoWDB.minimap and L["enabled"] or L["disabled"]))
    elseif msg:lower() == "hidetreasures" then
        local closest, dist = GetClosestTreasure()
        if closest then
            TreasuresOfWoWDB.hidden[closest.id] = true
            DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : Treasure "]..closest.name.." ("..closest.id..") "..L["is now hidden"])
        else
            DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : there's no treasure nearby"])
        end
    elseif msg:lower() == "clearhiddentreasures" then
    	table.wipe(TreasuresOfWoWDB.hidden)
    	DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : hidden treasures cleared"])
    elseif msg:lower() == "itm" then
        TreasuresOfWoWDB.ignoretm = not TreasuresOfWoWDB.ignoretm
        DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : treasure map quests now "]..(TreasuresOfWoWDB.ignoretm and L["ignored"] or L["used"]))
	elseif msg:lower() == "posxy" then
		local x, y, _, mapID = UnitPosition("player")
		local mapID2, isContinent = GetCurrentMapAreaID()
		local mapName = GetMapNameByID(mapID2)
		local GarrisonLvl = C_Garrison.GetGarrisonInfo(LE_GARRISON_TYPE_6_0)
		if GarrisonLvl ~= nil then GarrisonLvl = GarrisonLvl else GarrisonLvl = "0" end
		DEFAULT_CHAT_FRAME:AddMessage("x: "..x.." / y: "..y.." \nContinen_id = "..mapID.." \nMap - "..mapName.." = "..mapID2.."\nFaction : "..FactionGroup.."\nFief lvl "..GarrisonLvl)
    else
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /tow on|off")
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /tow debug")
		DEFAULT_CHAT_FRAME:AddMessage("Usage: /tow hidetreasures")
		DEFAULT_CHAT_FRAME:AddMessage("Usage: /tow clearhiddentreasures")
		DEFAULT_CHAT_FRAME:AddMessage("Usage: /tow posxy")
    end
end

local function WorldPosToScreenPos(x, y)
    local isMapDungeon = true
    local floorIndex, minY, minX, maxY, maxX = GetCurrentMapDungeonLevel()
    local zoneIndex, locLeft, locTop, locRight, locBottom

    if not (minX and minY and maxX and maxY) then
        isMapDungeon = false
        zoneIndex, locLeft, locTop, locRight, locBottom = GetCurrentMapZone()
        if not (locLeft and locTop and locRight and locBottom) then
            return 0, 0, false
        end
    end

    if isMapDungeon then
        local h, w = abs(maxY - minY), abs(maxX - minX)
        local sx = (maxX - x) / w
        local sy = (maxY - y) / h
        local onMap = y < maxY and y > minY and x < maxX and x > minX
        return sy, sx, onMap
    else
        local h, w = abs(locRight - locLeft), abs(locBottom - locTop)
        local sx = (locTop - x) / w
        local sy = (locLeft - y) / h
        local onMap = y < locLeft and y > locRight and x < locTop and x > locBottom
        return sy, sx, onMap
    end
end

function TOW:TomtomTreasure()
	local closest, dist = GetClosestTreasure()
	if closest then
		local x, y = WorldPosToScreenPos(closest.x, closest.y)
		local x = format("%.02f", x * 100)
		local y = format("%.02f", y * 100)
		local mapID = GetCurrentMapAreaID()
		TomTom:AddMFWaypoint( mapID, nil, x / 100, y / 100, {crazy = true, title = closest.name,})
	else
		DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfWoW : there's no treasure nearby"])
	end
end
	
local function TreasureMap_ResetPOI(button)
    button:SetWidth(32)
    button:SetHeight(32)
    button.Texture:SetWidth(16)
    button.Texture:SetHeight(16)
    button.Texture:SetPoint("CENTER", 0, 0)
    button.Texture:SetTexture("Interface\\Minimap\\POIIcons")
	if button.HighlightTexture then
        --button.HighlightTexture:SetWidth(16)
        --button.HighlightTexture:SetHeight(16)
        button.HighlightTexture:SetTexture("Interface\\Minimap\\POIIcons");
    end

    button.specialPOIInfo = nil
end

-- Hook the WorldMap OnClick
local function world_click_verify() 
	if TreasuresOfWoWDB.tomtomKey == "A" then return IsAltKeyDown()
	elseif TreasuresOfWoWDB.tomtomKey == "C" then return IsControlKeyDown()
	elseif TreasuresOfWoWDB.tomtomKey == "S" then return IsShiftKeyDown()
	end
end

local function TreasureMapPOI_OnClick(self, button)
    if button == "RightButton" then
        WorldMapButton_OnClick(WorldMapButton, button)
        return
    end
	if _G.TomTom and button == "LeftButton" and world_click_verify() then
		local t = TOW.treasures[self.id]
		local x, y, onMap = WorldPosToScreenPos(t.x, t.y)
		if onMap then
			local x = format("%.02f", x * 100)
			local y = format("%.02f", y * 100)
			local mapID = GetCurrentMapAreaID()
			TomTom:AddMFWaypoint( mapID, nil, x / 100, y / 100, {crazy = true, title = t.name,})
		end
    end
	if button == "LeftButton" and TreasuresOfWoWDB.debug then
		local debugMsgFormat = L["Treasure %s (%u):\nxpos : %.04f\nypos : %.04f\ntracking quest %u (%s)\ntreasuremap quest %u (%s)\ndistance %.02f"]
		local t = TOW.treasures[self.id]
		local trCheck, tmCheck, _, _, trquest, tmquest, _, _ = GetTreasureCompletion(t)
		local trstate = trquest ~= 0 and (trCheck and L["Not completed"] or L["Completed"]) or "No quest"
		local tmstate = tmquest ~= 0 and (tmCheck and L["Not completed"] or L["Completed"]) or "No quest"
		local x, y = UnitPosition("player")
		print(format(debugMsgFormat, t.name, t.id, t.x, t.y, trquest, trstate, tmquest, tmstate, GetDistanceToPoint(x, y, t.x, t.y)))
	end
end

local function TreasureMap_CreatePOI(index)
    local button = CreateFrame("Button", "TreasureMapFramePOI"..index, WorldMapButton)
    button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    button:SetScript("OnEnter", WorldMapPOI_OnEnter)
    button:SetScript("OnLeave", WorldMapPOI_OnLeave)
	button:SetScript("OnClick", TreasureMapPOI_OnClick)

    button.Texture = button:CreateTexture(button:GetName().."Texture", "BACKGROUND")
	button.HighlightTexture = button:CreateTexture(button:GetName().."HighlightTexture", "HIGHLIGHT");
    button.HighlightTexture:SetBlendMode("ADD");
    button.HighlightTexture:SetAlpha(.4);
    button.HighlightTexture:SetAllPoints(button.Texture);

    TreasureMap_ResetPOI(button)
    return button
end

local function DrawTreasurePOI()
    local worldMapID, isContinent = GetCurrentMapAreaID()
    local mapID = GetAreaMapInfo(worldMapID)
    local showTreasures = (not isContinent) and (worldMapID > 0) and TreasuresOfWoWDB.enabled

    for k,v in pairs(TOW.treasures) do
        local treasureMapPOI = _G["TreasureMapFramePOI"..k] or TreasureMap_CreatePOI(k)

        local trCheck, tmCheck, choise1Check, choise2Check, _, tmquest, c1quest, _ = GetTreasureCompletion(v)
		local factionTreasure = (v.faction == FactionGroup or v.faction == nil)
		local GarrisonLvl = C_Garrison.GetGarrisonInfo(LE_GARRISON_TYPE_6_0)
		local GarrisonLvlTreasure = (v.garrisonLvl == GarrisonLvl or v.garrisonLvl == nil)
		
        if showTreasures and (TreasureContinentsIds[mapID] == v.continent) and (TreasuresOfWoWDB.debug or (factionTreasure and GarrisonLvlTreasure and TreasuresOfWoWDB.mapIDhidden[v.mapID] and not TreasuresOfWoWDB.hidden[v.id] and trCheck and tmCheck and choise1Check and choise2Check)) then
			local x, y, onMap = WorldPosToScreenPos(v.x, v.y)
			if onMap then
			
				local name
				if TreasuresOfWoWDB.debug then
					if v.lvl == nil or v.lvl == "" then
						name = format("%s (%u, %.02f, %.02f )", v.name, v.id, x * 100, y * 100)
					else
						name = format("%s (%u, Lvl-%u, %.02f, %.02f )", v.name, v.id, v.lvl, x * 100, y * 100)
					end
				else
					if v.lvl == nil or v.lvl == "" then
						name = format("%s ( %.02f, %.02f )", v.name, x * 100, y * 100)
					else
						name = format("%s ( Lvl_%u, %.02f, %.02f )", v.name, v.lvl, x * 100, y * 100)
					end
				end
				
				local description = format(L["Treasures Of WoW : %s !"], TreasureQuestsDesc[c1quest] or TreasureQuestsDesc[tmquest] or "Unknown")

				--x = x * WorldMapButton:GetWidth()
				--y = -y * WorldMapButton:GetHeight()
				--treasureMapPOI:SetPoint("CENTER", "WorldMapButton", "TOPLEFT", x, y)
				WorldMapPOIFrame_AnchorPOI(treasureMapPOI, x, y, WorldMap_GetFrameLevelForLandmark(LE_MAP_LANDMARK_TYPE_NORMAL));

				TreasureMap_ResetPOI(treasureMapPOI)

				local x1, x2, y1, y2 = GetPOITextureCoords(v.icon or 197)

				treasureMapPOI.Texture:SetTexCoord(x1, x2, y1, y2)
                treasureMapPOI.HighlightTexture:SetTexCoord(x1, x2, y1, y2);

				treasureMapPOI.id = k
				treasureMapPOI.name = name
				treasureMapPOI.description = description.."\n"..v.desc
				treasureMapPOI.mapLinkID = nil
				treasureMapPOI.poiID = v.id
                treasureMapPOI.landmarkType = LE_MAP_LANDMARK_TYPE_NORMAL
				
				treasureMapPOI:Show()
			else
				treasureMapPOI:Hide()
			end
        else
            treasureMapPOI:Hide()
        end
    end
end

local function MyWorldMapFrame_OnUpdate(self, e)
    WorldMapFrame_OnUpdate(self, e)
    DrawTreasurePOI()
end

WorldMapFrame:SetScript("OnUpdate", MyWorldMapFrame_OnUpdate)
